/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.policy;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.command.MoveConnectorCommand;
import org.fortiss.tooling.base.ui.editpart.ElementEditPartBase;
import org.fortiss.tooling.base.ui.editpart.StickyConnectorEditPartBase;
import org.fortiss.tooling.base.ui.utils.ZoomUtils;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

/**
 * Layout policy for model elements, e.g. diagram nodes.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 24C47B8F82827769F3E89593ED821AD8
 */
public class ElementLayoutEditPolicy<T extends ILayoutedModelElement & INamedCommentedElement>
		extends LayoutEditPolicyBase {

	/** The edit part. */
	private final ElementEditPartBase<T> part;

	/** Constructor */
	public ElementLayoutEditPolicy(ElementEditPartBase<T> part) {
		this.part = part;
	}

	/** {@inheritDoc} */
	@Override
	protected EditPolicy createChildEditPolicy(EditPart child) {
		if (child instanceof StickyConnectorEditPartBase) {
			return new ConstrainedNonResizableEditPolicy<T>(part);
		}
		return new NonResizableEditPolicy();
	}

	/** {@inheritDoc} */
	@Override
	protected Command getMoveChildrenCommand(Request request) {
		if (request instanceof ChangeBoundsRequest) {
			ZoomUtils.fixChangeBoundRequest((ChangeBoundsRequest) request,
					ZoomUtils.determineZoom(getHost().getRoot()));
			return new MoveConnectorCommand((ChangeBoundsRequest) request);
		}
		return null;
	}
}
